/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author:		Bart Vanbrabant <bart dot vanbrabant at zoeloelip dot be>|
  +----------------------------------------------------------------------+
*/

/* $Id: $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "SAPI.h"
#include "php_htscanner.h"

ZEND_DECLARE_MODULE_GLOBALS(htscanner)

#define FILE_BUFFER 1000

/* {{{ debug */
static void debug(char *format, ...) {
	char output_buf[512];
	va_list args;
	char *debug;

	debug = getenv("PHP_HTSCANNER_DEBUG");
	if (debug == NULL)
		return;
		
	va_start (args, format);
	vsnprintf (output_buf, sizeof (output_buf), format, args);
	va_end (args);
		
	fputs (output_buf, stderr);
	fflush (stderr);
}
/* }}} */

/* {{{ strip
 * Strip the leading whitespaces en tabs
 */
static char* strip(char* str) {
	char *p = str;
	/* find the first not whitespace or tab character */
	while ((*p == ' ' || *p == '\t') && *p != '\0') {
		++p;
	}
	return p;
}
/* }}} */

/* {{{ value_hnd
 * Parse an option and try to set the option 
 */
static void value_hnd(char *string, int flag, int status) {
	char *name, *value;
	
	/* strip any leading whitespaces or tabs from the name */
	name = strip(string);
	value = strchr(name, ' ');
	
	if (value) {
		int len;
		*value = 0;
		++value;
		
		/* strip any leading whitespaces or tabs from the value */
		len = strlen(value);
		value = strip(value);

		if (value[len - 2] == '\r') {
			value[len - 2] = 0;
		} else {
			value[len - 1] = 0;
		}

		if (flag) { /* it's a flag */
			if (!strcasecmp(value, "On") || 
					(value[0] == '1' && value[1] == '\0')) {
				value = "1";
			} else {
				value = "0";
			}
		} else { /* it's a value */
			if (!strncasecmp(value, "none", sizeof("none"))) {
				value = "";
			}
		}
		debug("\tParsed: %s = %s\n", name, value);

		/* alter the ini settings v1 */
		if (zend_alter_ini_entry(name, strlen(name) + 1, value, strlen(value), 
					status, PHP_INI_STAGE_RUNTIME) == FAILURE) {
			zend_error(E_ERROR, "Adding option (Name: %s Value: %s) failed!\n", name, value);
		}

		/* {{{ v2 */
#if 0 //TODO: safemode support
#define _CHECK_PATH(var, ini) php_ini_check_path(Z_STRVAL_PP(var), Z_STRLEN_PP(var), ini, sizeof(ini))

		/* safe_mode & basedir check */
		if (PG(safe_mode) || PG(open_basedir)) {
			if (_CHECK_PATH(name, "error_log") ||
					_CHECK_PATH(name, "java.class.path") ||
					_CHECK_PATH(name, "java.home") ||
					_CHECK_PATH(name, "java.library.path") ||
					_CHECK_PATH(name, "session.save_path") ||
					_CHECK_PATH(name, "vpopmail.directory")) {
				if (PG(safe_mode) &&(!php_checkuid(Z_STRVAL_PP(value), NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
					zval_dtor(return_value);
					RETURN_FALSE;
				}

				if (php_check_open_basedir(Z_STRVAL_PP(value) TSRMLS_CC)) {
					zval_dtor(return_value);
					RETURN_FALSE;
				}
			}
		}

		/* checks that ensure the user does not overwrite certain ini settings when safe_mode is enabled */
		if (PG(safe_mode)) {
			if (!strncmp("max_execution_time", Z_STRVAL_PP(name), sizeof("max_execution_time")) ||
					!strncmp("memory_limit", Z_STRVAL_PP(name), sizeof("memory_limit")) ||
					!strncmp("child_terminate", Z_STRVAL_PP(name), sizeof("child_terminate"))) {
				zval_dtor(return_value);
				RETURN_FALSE;
			}
		}
#endif
		/* }}}*/
	}
}
/* }}} */

/* {{{ parse_config_file
 * Parse the configuration file
 */
static void parse_config_file(char *file) {
	FILE *f;
	char str[FILE_BUFFER];
	char *opt;
	char *pos;
	
	f = fopen(file, "r");
	if (f != NULL) {
		while (fgets(str, FILE_BUFFER, f) != NULL) {
			/* strip leading spaces or tabs */
			pos = strip(str);
			opt = NULL;
			if (strncmp(pos, "php_value", sizeof("php_value") - 1) == 0) {
				value_hnd(pos + sizeof("php_value"), 0, PHP_INI_PERDIR);
			} 
			else if (strncmp(pos, "php_flag", sizeof("php_flag") - 1) == 0) {
				value_hnd(pos + sizeof("php_flag"), 1, PHP_INI_PERDIR);
			} 
		}
		fclose(f);
	}

}
/* }}} */

/* {{{ get_doc_root
 * doc_root only seems to be available in _SERVER["doc_root"] 
 * so get it there.
 */
static char* get_doc_root(TSRMLS_DC) {
	zval **server, **data;
	
	if (zend_hash_find(&EG(symbol_table), "_SERVER", sizeof("_SERVER"), 
				(void **) &server) != FAILURE && (Z_TYPE_PP(server)==IS_ARRAY)) {
		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(server));
		if (zend_hash_find(Z_ARRVAL_PP(server), "DOCUMENT_ROOT", 
				sizeof("DOCUMENT_ROOT"), (void **) &data) != FAILURE) {
			if (Z_TYPE_PP(data) == IS_STRING) {
				return Z_STRVAL_PP(data);
			}
		}
	}
	return NULL;
}
/* }}} */

/* True global resources - no need for thread safety here */
static int le_htscanner;

/* {{{ htscanner_functions[]
 *
 */
function_entry htscanner_functions[] = {
	{NULL, NULL, NULL}	/* Must be the last line in htscanner_functions[] */
};
/* }}} */

/* {{{ htscanner_module_entry
 */
zend_module_entry htscanner_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"htscanner",
	htscanner_functions,
	PHP_MINIT(htscanner),
	PHP_MSHUTDOWN(htscanner),
	PHP_RINIT(htscanner),
	PHP_RSHUTDOWN(htscanner),
	PHP_MINFO(htscanner),
#if ZEND_MODULE_API_NO >= 20010901
	HTSCANNER_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_HTSCANNER
ZEND_GET_MODULE(htscanner)
#endif

/* {{{ PHP_INI
 */
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("htscanner.config_file", ".htaccess", PHP_INI_SYSTEM, OnUpdateString, config_file, zend_htscanner_globals, htscanner_globals)
	STD_PHP_INI_ENTRY("htscanner.default_docroot", "/", PHP_INI_SYSTEM, OnUpdateString, default_docroot, zend_htscanner_globals, htscanner_globals)
PHP_INI_END()
/* }}} */

/* {{{ php_htscanner_init_globals
 */
static void php_htscanner_init_globals(zend_htscanner_globals *htscanner_globals)
{
	htscanner_globals->config_file = NULL;
	htscanner_globals->default_docroot = NULL;
}
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(htscanner)
{
	ZEND_INIT_MODULE_GLOBALS(htscanner, php_htscanner_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(htscanner)
{
	UNREGISTER_INI_ENTRIES();
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(htscanner)
{
	char *doc_root, *script_name;

	doc_root = get_doc_root(TSRMLS_C);
	if (doc_root == NULL) {
		doc_root = HTG(default_docroot);
	}
	script_name = SG(request_info).path_translated;
	
	debug("\nStarting request ... \n");
	
	if (script_name != NULL && doc_root != NULL) {
		size_t i, len, ht_len, tmp;
		len = strlen(script_name);
		ht_len = strlen(HTG(config_file));
		for (i = strlen(doc_root); i < len; i++) {
			if (script_name[i] == PHP_DIR_SEPARATOR) {
				char file[FILE_BUFFER];
				tmp = i + 1 + ht_len; /* + 1 for trailing slash */
				memset(file, 0, tmp);
				strncpy(file, script_name, i + 1);
				/* add a trailing 0 */
				file[i + 1] = '\0';
				strcat(file, HTG(config_file));
				debug("Going to parse: %s\n", file);
				parse_config_file(file);
				debug("Done parsing: %s (%d = %d)\n", file, strlen(file), tmp);
			}
		}
	}
	debug("Finished iterating ... \n");
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(htscanner)
{
	debug("Request cleaned up!\n");
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(htscanner)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "htscanner support", "enabled");
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
